# -*- coding: utf-8 -*-

#
# ini7 Channel Loader
#
# Coded by dre (c) 2024
# Support: boxpirates.to
# E-Mail: dre@drecomx.net
#
# This plugin is open source but it is NOT free software.
#
# This plugin may only be distributed to and executed on hardware which
# is licensed by Dream Property.
# In other words:
# It's NOT allowed to distribute any parts of this plugin or its source code in ANY way
# to hardware which is NOT licensed by Dream Property.
# It's NOT allowed to execute this plugin and its source code or even parts of it in ANY way
# on hardware which is NOT licensed by Dream Property.
#
# If you want to use or modify the code or parts of it,
# you have to keep MY license and inform me about the modifications by mail.

from Components.ActionMap import ActionMap, HelpableActionMap, NumberActionMap
from Components.ConfigList import ConfigListScreen
from Components.config import config, ConfigSubsection, ConfigYesNo, ConfigText, ConfigSet, ConfigSubList, ConfigSelection, ConfigInteger, getConfigListEntry, NoSave, configfile
from Components.FileList import FileList
from Components.Input import Input
from Components.Label import Label
from Components.Pixmap import Pixmap
from Components.Sources.List import List
from Screens.ChannelSelection import MODE_TV
from Screens.ChoiceBox import ChoiceBox
from Screens.HelpMenu import HelpableScreen
from Screens.InfoBar import InfoBar
from Screens.InputBox import InputBox
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from ServiceReference import ServiceReference
from Plugins.Plugin import PluginDescriptor
from Tools.Directories import fileExists, resolveFilename, SCOPE_CURRENT_SKIN
from Tools.LoadPixmap import LoadPixmap
from twisted.web.client import getPage, downloadPage
from urllib import quote
from enigma import eServiceReference, eServiceCenter, eDVBDB, eListboxPythonMultiContent, eListbox, gFont, \
	RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_VALIGN_CENTER
from os import system as os_system
import re
import xml.etree.cElementTree as ET
from xml.dom import minidom
import json

from . import _

config.plugins.init7channelloader = ConfigSubsection()
config.plugins.init7channelloader.url = ConfigText(default="https://api.init7.net/tvchannels.m3u", fixed_size=False)
config.plugins.init7channelloader.countries = ConfigSubList()
config.plugins.init7channelloader.countrycount = ConfigInteger(0)
config.plugins.init7channelloader.addmarker = ConfigYesNo(default=True)
config.plugins.init7channelloader.generatexml = ConfigYesNo(default=False)

code2country = {
	'chde': "%s (%s)" %(_("Switzerland"), _("German")),
	'de': _("Germany"),
	'at': _("Austria"),
	'chfr': "%s (%s)" %(_("Switzerland"), _("French")),
	'fr': _("France"),
	'chit': "%s (%s)" %(_("Switzerland"), _("Italian")),
	'it': _("Italy"),
	'uk': _("England"),
	'us': _("USA"),
	'es': _("Spain"),
	'pt': _("Portugal"),
	'nl': _("Netherlands"),
	'ro': _("Romania"),
	'me': _("Montenegro"),
	'rs': _("Serbia"),
	'pl': _("Poland"),
	'hu': _("Hungary"),
	'ba': _("Bahrain"),
	'gr': _("Greece"),
	'al': _("Albania"),
	'ae': _("United Arab Emirates"),
	'cn': _("China"),
	'tw': _("Taiwan"),
	'tr': _("Turkey"),
	'ar': _("Argentina"),
	'qa': _("Qatar"),
}

def initConfig():
	count = config.plugins.init7channelloader.countrycount.value
	if count != 0:
		i = 0
		while i < count:
			country = ConfigText(default = "")
			config.plugins.init7channelloader.countries.append(country)
			i += 1
			pass

initConfig()

def Plugins(**kwargs):
    return [PluginDescriptor(name="Init7 Channel Loader", description=_("Get multicast channels for init7"), where = [PluginDescriptor.WHERE_PLUGINMENU], fnc=openInit7ChanneLoader, )]
    
def openInit7ChanneLoader(session, **kwargs):
	print kwargs
	session.open(Init7ChannelLoader)
	
EXTINFO = re.compile('#EXTINF:0 tvg-logo=\"[A-Za-z0-9:\/\.\-_]*\".*group-title=\"(.*?)\", (.*?)\n')
SELECTEDCHANNELS = "/usr/lib/enigma2/python/Plugins/Extensions/init7channelloader/selectedchannels.txt"
SATMAPPING = "/usr/lib/enigma2/python/Plugins/Extensions/init7channelloader/satmapping.txt"
EPGMAPPING = "/usr/lib/enigma2/python/Plugins/Extensions/init7channelloader/epgmapping.txt"
CHANNELXML = "/etc/enigma2/channels_fiber.xml"
	
class Init7ChannelLoader(Screen, HelpableScreen):
	skin = """
		<screen name="Init7ChannelLoader" position="center,center" size="1140,670" title="Init7 Channel Loader">
			<widget font="Regular;25" halign="center" valign="center" name="info" position="10,10" size="1020,30"/>
			<widget source="list" render="Listbox" position="40,50" size="1060,540" scrollbarMode="showOnDemand" backgroundColor="#000000" zPosition="2"  backgroundColorSelected="#a90000" foregroundColorSelected="#00ffffff">
				<convert type="TemplatedMultiContent">
					{"templates":
						{"default":
							(45,
								[
									MultiContentEntryPixmapAlphaBlend(pos=(10,10), size=(25,25), png=4),
									MultiContentEntryText(pos=(45,0), size=(650,45), flags=RT_WRAP|RT_VALIGN_CENTER, font=0, text=0),
									MultiContentEntryText(pos=(695,0), size=(350,45), flags=RT_WRAP|RT_VALIGN_CENTER, font=0, text=2),
								]),
						},
					"fonts": [gFont("Regular",30)]
					}
				</convert>				
			</widget>
			<ePixmap alphatest="blend" pixmap="skin_default/buttons/red.png" position="10,630" zPosition="3" size="30,30" transparent="1" name="red" />
			<widget name="ButtonRedText" position="50,630" size="200,30" valign="center" halign="left" font="Regular;25" transparent="1" />
			<ePixmap alphatest="blend" pixmap="skin_default/buttons/green.png" position="270,630" zPosition="3" size="30,30" transparent="1" name="green" />
			<widget name="ButtonGreenText" position="310,630" size="200,30" valign="center" halign="left" font="Regular;25" transparent="1" />
			<ePixmap alphatest="blend" pixmap="skin_default/buttons/yellow.png" position="530,630" zPosition="3" size="30,30" transparent="1" name="yellow" />
			<widget name="ButtonYellowText" position="570,630" size="200,30" valign="center" halign="left" font="Regular;25" transparent="1" />
			<ePixmap alphatest="blend" pixmap="skin_default/buttons/blue.png" position="790,630" zPosition="3" size="30,30" transparent="1" name="blue" />
			<widget name="ButtonBlueText" position="830,630" size="200,30" valign="center" halign="left" font="Regular;25" transparent="1" />
			<eLabel font="Regular;25" position="1050,630" size="80,30" text="MENU" transparent="0" valign="center" halign="center" foregroundColor="#ffffff" backgroundColor="#787878" zPosition="6"/>	
		</screen>	
	"""
	
	def __init__(self, session, args=None):
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"ok":		(self.selectEntry, _("Select entry")),
			"back":	(self.close, _("Close")),
			
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"red":		(self.openFilterAndSort, _("Filter and sort countries")),
			"green":	(self.selectFile, _("Select file for conversion")),
			"yellow":	(self.downloadM3U, _("Download file for conversion")),
			"blue":		(self.showBouquetInputBox, _("Create bouquet with selected channels")),
		}, -1)
		
		self["MenuActions"] = HelpableActionMap(self, "MenuActions",
		{
			"menu":	(self.openConfig, _("Show configuration")),
			
		}, -1)
		
		"""
		self["ChannelSelectEPGActions"] = HelpableActionMap(self, "ChannelSelectEPGActions",
		{
			"showEPGList":	(self.openFilterAndSort, _("Filter and sort countries")),
			
		}, -1)
		"""
		
		self["list"] = List()
		self["info"] = Label(_("Press green to select or yellow to download m3u file"))
		self["list"].setBuildFunc(self.buildServiceListEntry)

		self["ButtonRed"] = Pixmap()
		self["ButtonRedText"] = Label()

		self["ButtonGreen"] = Pixmap()
		self["ButtonGreenText"] = Label(_("Select File"))

		self["ButtonYellow"] = Pixmap()
		self["ButtonYellowText"] = Label(_("Download File"))

		self["ButtonBlue"] = Pixmap()
		self["ButtonBlueText"] = Label()
		
		self.selectedList = []
		if fileExists(SELECTEDCHANNELS):
			with open(SELECTEDCHANNELS) as file:
				self.selectedList = [x.strip() for x in file.readlines()]
		
		self.satmapping = {}		
		if fileExists(SATMAPPING):
			with open(SATMAPPING) as file:
				data = file.read()
				
			self.satmapping = json.loads(data)

		self.epgmapping = {}		
		if fileExists(EPGMAPPING):
			with open(EPGMAPPING) as file:
				data = file.read()
				
			self.epgmapping = json.loads(data)			
	
		self.countryList = []
		self.countryAndCodeList = []
		self.currentFilename = None
		
		self.checkForXml()

	# todo: prettyprint
	def updateChannelsXml(self):
		if len(self.channellist):
			tree = ET.parse(CHANNELXML)
			root = tree.getroot()
			entryList = []
			if root.tag == "channels":
				for entry in root.itertext():
					entryList.append(entry)
				for channel in self.channellist:
					if channel[0] in entryList:
						continue
					else:
						child = ET.SubElement(root, "channel")
						child.text = channel[0]
						child.set("blacklisted", "1")
						child.set("id", "")
						child.set("sref", "")
						tree.write(CHANNELXML)

	def addSrefToXml(self, sref=None, channelname=None, epgId="", mappedToDvb=False):
		if sref is not None:
			tree = ET.parse(CHANNELXML)
			root = tree.getroot()
			
			if root.tag == "channels":
				writeToFile = False
				channelFound = False
				for channel in root.iter("channel"):
					if channel.text	== channelname:
						channelFound = True
						currentRef = channel.get("sref", "")
						# current sref does not match calculated sref --> update required?
						if currentRef != sref:
							if currentRef != "":
								currentRefSplit = currentRef.split(":")
								# no dvbnamespace set means no dvb-service mapped to channel						
								if currentRefSplit[6]=="0":
									currentRef = ""
							# ok, we can update
							if  currentRef == "":
								channel.set("sref", sref)
								writeToFile = True
						currentEpgId = channel.get("id", "")
						if currentEpgId != epgId:
							channel.set("id", epgId)
							writeToFile = True
							
						blacklisted = channel.get("blacklisted", "")
						
						# channels without provided EPG or with corresponding DVB channel must be excluded from load
						if blacklisted == "0" and (epgId == "" or mappedToDvb):
							channel.set("blacklisted", "1")
							writeToFile = True
						break

			if not channelFound:
				channel = ET.SubElement(root, "channel")
				channel.set('sref', sref)
				channel.set('id', epgId)
				channel.text = channelname
				blacklisted = "0"
				if epgId == "" or mappedToDvb:
					blacklisted = "1"
				channel.set('blacklisted', blacklisted)
				writeToFile = True

						
			if writeToFile:
				tree.write(CHANNELXML)
				return True
		return False
	
	def prettify(self, elem):
		raw = ET.tostring(elem, 'utf-8')
		pretty = minidom.parseString(raw)
		return pretty.toprettyxml(indent="\t")

	def openConfig(self):
		self.session.openWithCallback(self.checkForXml, Init7ChannelLoaderConfig)
		
	def checkForXml(self, deleteFile=False):
		if config.plugins.init7channelloader.generatexml.value:
			if deleteFile:
				os_system("rm %s" %(CHANNELXML))
				
			if not fileExists(CHANNELXML):
				os_system("touch %s" %(CHANNELXML))
				root = ET.Element("channels")
				tree = ET.ElementTree(root)
				tree.write(CHANNELXML)

	def showBouquetInputBox(self):
		if self["list"].count():
			self.session.openWithCallback(self.bouquetInputCallback, InputBox, title=_("Please enter a name for the new bouquet"), text="Init7", maxSize=False, visible_width = 56, type=Input.TEXT)

	def bouquetInputCallback(self, bouquet):
		if bouquet is not None:
			self.createBouquet(bouquet)

	def createBouquet(self, bName):
		mode = MODE_TV
		services = self.addSelectedServicesToBouquet()
		if config.usage.multibouquet.value:
			mutableBouquetList = self.getMutableBouquetList(mode)
			if mutableBouquetList:
				if mode == MODE_TV:
					bName += " (TV)"
					finalName = self.buildBouquetID(bName, "userbouquet.", mode)
					sref = '1:7:1:0:0:0:0:0:0:0:FROM BOUQUET \"userbouquet.%s.tv\" ORDER BY bouquet'%(finalName)
				else:
					bName += " (Radio)"
					finalName = self.buildBouquetID(bName, "userbouquet.", mode)
					sref = '1:7:2:0:0:0:0:0:0:0:FROM BOUQUET \"userbouquet.%s.radio\" ORDER BY bouquet'%(finalName)
				new_bouquet_ref = eServiceReference(sref)
				if not mutableBouquetList.addService(new_bouquet_ref):
					mutableBouquetList.flushChanges()
					eDVBDB.getInstance().reloadBouquets()
					mutableBouquet = self.getMutableList(new_bouquet_ref)
					if mutableBouquet:
						mutableBouquet.setListName(bName)
						if services is not None:
							for service in services:
								if mutableBouquet.addService(service):
									self["info"].setText(_("add", service.toString(), "to new bouquet failed"))
						mutableBouquet.flushChanges()
						self.setRoot(self.bouquet_rootstr)
						self["info"].setText(_("Bouquet created"))
					else:
						self["info"].setText(_("Get mutable list for new created bouquet failed!"))
				else:
					self["info"].setText(_("Bouquet %s already exists." % bName))
			else:
				self["info"].setText(_("Bouquet is not editable!"))
		else:
			self["info"].setText(_("Multiple bouquet is not enabled in settings!"))

	def getMutableBouquetList(self, mode):
		if mode == MODE_TV:
			self.bouquet_rootstr = '1:7:1:0:0:0:0:0:0:0:FROM BOUQUET "bouquets.tv" ORDER BY bouquet'
		else:
			self.bouquet_rootstr = '1:7:1:0:0:0:0:0:0:0:FROM BOUQUET "bouquets.radio" ORDER BY bouquet'
		return self.getMutableList(eServiceReference(self.bouquet_rootstr))

	def getMutableList(self, ref):
		serviceHandler = eServiceCenter.getInstance()
		return serviceHandler.list(ref).startEdit()

	def buildBouquetID(self, str, prefix, mode):
		tmp = str.lower()
		name = ''
		for c in tmp:
			if (c >= 'a' and c <= 'z') or (c >= '0' and c <= '9'):
				name += c
			else:
				name += '_'
		# check if file is unique
		suffix = ""
		if mode == MODE_TV:
			suffix = ".tv"
		else:
			suffix = ".radio"
		filename = '/etc/enigma2/' + prefix + name + suffix
		if fileExists(filename):
			i = 1
			while True:
				filename = "/etc/enigma2/%s%s_%d%s" %( prefix , name , i, suffix)
				if fileExists(filename):
					i += 1
				else:
					name = "%s_%d" % (name,i)
					break
		return name

	def setRoot(self, bouquet_rootstr):
		infoBarInstance = InfoBar.instance
		if infoBarInstance is not None:
			servicelist = infoBarInstance.servicelist
			root = servicelist.getRoot()
			if bouquet_rootstr ==  root.toString():
				currentref = servicelist.getCurrentSelection()
				servicelist.setRoot(root)
				servicelist.setCurrentSelection(currentref)

	def addSelectedServicesToBouquet(self, bouquet=None):
		servicesList = []
		xmlEntryList = []
		serviceGroup = ""
		cnt = 0
		counter = 1
		#onidDict = {"0":"1F4", "1":"1F5", "2":"1F6", "3":"1F7", "4":"1F8", "5":"1F9", "6":"1FA", "7":"1FB", "8":"1FC", "9":"1FD", "10":"1FE", "11":"1FF"}
		onidDict = {"0":"FFF4", "1":"FFF5", "2":"FFF6", "3":"FFF7", "4":"FFF8", "5":"FFF9", "6":"FFFA", "7":"FFFB", "8":"FFFC", "9":"FFFD", "10":"FFFE", "11":"FFFF"}
		
		rewriteXML = False
		
		self.checkForXml(True)			
		
		for service in self["list"].list:
			if service[0] in self.selectedList:
				if serviceGroup != service[1]:
					serviceGroup = service[1]
					markerName = service[1]
					if code2country.get(service[1], None) is not None:
						markerName = code2country.get(service[1], None)
					sRef = "1:64:%d:0:0:0:0:0:0:0::%s" %(cnt, markerName)
					ref = eServiceReference(sRef)
					servicesList.append(ref)
					cnt += 1
				sRef = "1:256:19:0:0:0:0:0:0:0:%s" %(quote(service[3]))
				if config.plugins.init7channelloader.generatexml.value:
					# use IP address of stream to build a random ONID and SID
					# this guarantees a unique sref for EPG look up
					# ONID (group 3): 0 = FFF4, 1 = FFF5, 2 = FFF6, 3 = FFF7, 4 = FFF8, 5 = FFF9
					# SID (group 4): 0 - 255 converted to hex
					# use IP-address to build sref
					mappedDvbSref = self.satmapping.get(service[0], None)
					mappedToDvb = False
					if  mappedDvbSref is None:
						ipdata = service[3].split(":")
						ip = ipdata[1].split(".")
						onid = onidDict.get(ip[2], "0")
						sid = '{:x}'.format(int(ip[3]))
					
						sRef = "1:256:19:%s:0:%s:0:0:0:0:%s" %(sid, onid, quote(service[3]))
					else:
						mappedToDvb = True
						print "%s is mapped to %s" %(quote(service[3]), mappedDvbSref)
						sRef = "%s:%s" %(mappedDvbSref.encode('utf-8', 'ignore'), quote(service[3]))
					
					result = self.addSrefToXml(sRef, service[0], self.epgmapping.get(service[0], ""), mappedToDvb)
					if result:
						rewriteXML = True

				sName = service[0]
				ref = eServiceReference(sRef)
				if sName:
					ref.setName(sName)
				servicesList.append(ref)
		
		if rewriteXML:
			tree = ET.parse(CHANNELXML)
			root = tree.getroot()
			x = self.prettify(root)
			with open(CHANNELXML, "w") as f:
				f.write(x)					
		
		return servicesList

	def downloadM3U(self):
		d = downloadPage(config.plugins.init7channelloader.url.value, file='/tmp/channels.m3u', method='GET')
		
		def downloadFinished(result):
			self.parseM3U('/tmp/channels.m3u')
			
		d.addCallback(downloadFinished)
		
		def showError(error):
			self.session.open(MessageBox, _("Error while downloading file"), MessageBox.TYPE_ERROR, 5)
			
		d.addErrback(showError)
			
	def buildServiceListEntry(self, channelName, channelGroup, channelCountry, url):
		if channelName in self.selectedList:
			png = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, "skin_default/icons/lock_on.png"))
		else:
			png = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, "skin_default/icons/lock_off.png"))
		return [channelName, channelGroup, channelCountry, url, png]

	def selectEntry(self):
		if len(self["list"].list)>0:
			current = self["list"].getCurrent()
			index = self["list"].getIndex()
		
			if current[0] in self.selectedList:
				self.selectedList.remove(current[0])
			else:
				self.selectedList.append(current[0])
			self["list"].modifyEntry(index, current)		
		
	def selectFile(self):
		self.session.openWithCallback(self.parseM3U, Init7ChannelLoaderFileSelector)
		
	def parseM3U(self, filename=None):
		templist = []
		self.countryAndCodeList = []
		
		if filename is not None:
			self.currentFilename = filename
		elif filename is None and self.currentFilename is not None:
			filename = self.currentFilename
		
		# init sort list with my default
		defaultsortlist = ['chde', 'de', 'at', 'chfr', 'fr', 'chit', 'it', 'uk', 'us', 'es', 'pt', 'nl', 'ro', 'me', 'rs', 'pl', 'hu', 'ba', 'gr', 'al', 'sh', 'ae', 'cn', 'tw', 'tr', 'ar', 'qa' ]
		
		sortlist = []
		
		for i in range(len(config.plugins.init7channelloader.countries)):
			if config.plugins.init7channelloader.countries[i].value != "none":
				sortlist.append(config.plugins.init7channelloader.countries[i].value)
				
		if len(sortlist) == 0:
			sortlist = defaultsortlist[:]
		
		countryChannelDict = {}
		self.channellist = []
		if filename is not None:
			f = open(filename, "r")
			for line in f:
				tvdata = EXTINFO.match(line)
				if tvdata is not None:
					templist.extend([tvdata.groups()[1]])
					currentCountry = tvdata.groups()[0].strip()
					templist.extend([tvdata.groups()[0], code2country.get(currentCountry, currentCountry)])
					
					if not code2country.get(currentCountry, currentCountry) in self.countryList:
						self.countryList.append(code2country.get(currentCountry, currentCountry))
						self.countryAndCodeList.append((currentCountry, code2country.get(currentCountry, currentCountry)))
				elif line.startswith("udp:"):
					templist.extend([line.strip("\n")])
					currentChannelList = countryChannelDict.get(currentCountry, [])
					currentChannelList.append(templist)
					countryChannelDict[currentCountry] = currentChannelList
					templist = []
			f.close()
			
			for country in sortlist:
				channels = countryChannelDict.get(country, [])
				for channel in channels:
					self.channellist.append(tuple(channel))
			
			self["info"].setText(_("Select services to import:"))
		else:
			self["info"].setText(_("Press green to select or yellow to download m3u file"))
		self["list"].setList(self.channellist)
		
		self.setText()

	def setText(self):
		if self["list"].count():
			self["ButtonBlueText"].setText(_("Create Bouquet"))
			self["ButtonRedText"].setText(_("Filter & sort"))

	def openFilterAndSort(self):
		if not len(self.countryList):
			self.session.open(MessageBox, _("Please download channels first to apply sorting and filtering"), MessageBox.TYPE_ERROR, timeout=5)
		else:
			self.session.openWithCallback(self.parseM3U, Init7ChannelLoaderFilterAndSort, self.countryAndCodeList)
		
class Init7ChannelLoaderFileSelector(Screen, HelpableScreen):
	skin = """
		<screen name="Init7ChannelLoaderFileSelector" position="center,center" size="800,460" title="Select M3U File">
			<widget name="filelist" position="10,10" scrollbarMode="showOnDemand" size="780,400" />
			<eLabel font="Regular;20" position="10,420" size="60,30" text="OK" transparent="0" valign="center" halign="center" foregroundColor="#ffffff" backgroundColor="#787878" zPosition="6"/>
			<eLabel font="Regular;20" position="80,420" size="60,30" text="EXIT" transparent="0" valign="center" halign="center" foregroundColor="#ffffff" backgroundColor="#787878" zPosition="6"/>
		</screen>
	"""
	
	def __init__(self, session, args=None):
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)

		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"ok":	(self.changeDir, _("Change directory / select file")),
			"back":	(self.close, _("Close")),
			
		}, -1)
	
	
		self["filelist"] = FileList("/tmp/", showDirectories=True, showFiles=True, showMountpoints=False, matchingPattern = "^.*\.m3u", useServiceRef = False)
		
		self.setTitle(_("Select M3U File"))
		
	def changeDir(self):
		if self["filelist"].canDescent():
			self["filelist"].changeDir(self["filelist"].getFilename())
		else:
			self.close(self["filelist"].getCurrentDirectory()+"/"+self["filelist"].getFilename())
			
class Init7ChannelLoaderConfig(Screen, HelpableScreen, ConfigListScreen):
	skin = """
			<screen position="center,center" size="1240,240" title="Init7 Channel Loader - Config">
				<widget name="config" position="10,10" size="1220,180" scrollbarMode="showOnDemand" separation="450" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/red.png" position="10,200" zPosition="3" size="30,30" transparent="1" name="red" />
				<widget name="ButtonRedText" position="50,200" size="400,30" valign="center" halign="left" font="Regular;25" transparent="1" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/green.png" position="610,200" zPosition="3" size="30,30" transparent="1" name="green" />
				<widget name="ButtonGreenText" position="650,200" size="400,30" valign="center" halign="left" font="Regular;25" transparent="1" />
			</screen>	
	"""
	
	def __init__(self, session, args=None):
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"ok":		(self.keySave, _("Save settings")),
			"back":		(self.keyCancel, _("Close")),
			
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.keySave, _("Save settings")),
			"red":		(self.keyCancel, _("Cancel")),
		}, -1)
	
		self["ButtonGreen"] = Pixmap()
		self["ButtonGreenText"] = Label(_("Save settings"))
		self["ButtonRed"] = Pixmap()
		self["ButtonRedText"] = Label(_("Cancel"))
		
		self.list = []
		
		ConfigListScreen.__init__(self, self.list)
		
		self.list.append(getConfigListEntry(_("Download URL:"), config.plugins.init7channelloader.url))
		self.list.append(getConfigListEntry(_("Add markers for channel groups:"), config.plugins.init7channelloader.addmarker))
		self.list.append(getConfigListEntry(_("Generate xml-file for EPGImporter:"), config.plugins.init7channelloader.generatexml))
		
		self["config"].setList(self.list)
		
		self.setTitle(_("Init7 Channel Loader - Configuration"))
		
class Init7ChannelLoaderFilterAndSort(Screen, HelpableScreen, ConfigListScreen):
	skin = """
			<screen position="center,center" size="1240,650" title="Init7 Channel Loader - Filter and Sort">
				<widget name="config" position="0,0" size="1240,600" scrollbarMode="showOnDemand" separation="70" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/red.png" position="10,610" zPosition="3" size="30,30" transparent="1" name="red" />
				<widget name="ButtonRedText" position="50,610" size="250,30" valign="center" halign="left" font="Regular;25" transparent="1" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/green.png" position="310,610" zPosition="3" size="30,30" transparent="1" name="green" />
				<widget name="ButtonGreenText" position="350,610" size="250,30" valign="center" halign="left" font="Regular;25" transparent="1" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/yellow.png" position="610,610" zPosition="3" size="30,30" transparent="1" name="yellow" />
				<widget name="ButtonYellowText" position="650,610" size="250,30" valign="center" halign="left" font="Regular;25" transparent="1" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/blue.png" position="910,610" zPosition="3" size="30,30" transparent="1" name="blue" />
				<widget name="ButtonBlueText" position="950,610" size="250,30" valign="center" halign="left" font="Regular;25" transparent="1" />
			</screen>	
	"""
	
	def __init__(self, session, countryAndCodeList, args=None):
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self.session = session
		self.countryAndCodeList = countryAndCodeList
		
		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"ok":		(self.keySave, _("Save settings")),
			"back":		(self.keyCancel, _("Close")),
			
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.keySave, _("Save settings")),
			"red":		(self.keyCancel, _("Cancel")),
			"yellow":	(self.removeOrChangeEntry, _("Remove or change country")),
			"blue":		(self.addEntry, _("Add country")),
		}, -1)
	
		self["ButtonGreenText"] = Label(_("Save settings"))
		self["ButtonRedText"] = Label(_("Cancel"))
		self["ButtonYellowText"] = Label(_("Remove") + "/" + _("Change"))
		self["ButtonBlueText"] = Label(_("Add"))
		
		self.list = []
		
		# without session as we don't need an input help!
		ConfigListScreen.__init__(self, self.list)
		
		self["config"].setList(self.list)
		self.createConfig()
		
		self.setTitle(_("Init7 Channel Loader - Filter and Sort"))

	def addEntry(self):
		self.session.openWithCallback(self.createConfigEntry, Init7ChannelLoaderCountrySelector, self.countryAndCodeList)
		
	def createConfigEntry(self, retValue):
		if retValue:
			self.createConfig()
		
	def createConfig(self):
		self.list = []
		count = config.plugins.init7channelloader.countrycount.value
		if count != 0:
			i = 0
			while i < count:
				entry = ConfigText(default = code2country.get(config.plugins.init7channelloader.countries[i].value, config.plugins.init7channelloader.countries[i].value ))
				self.list.append(getConfigListEntry(_("Country"), entry))
				i += 1
					
		self["config"].setList(self.list)
	
	def removeOrChangeEntry(self):
		self.session.openWithCallback(self.actionSelected, ChoiceBox, title=_("Select action"), list = ( (_("Change"), "change"), (_("Remove"), "remove")))
		
	def actionSelected(self, answer):
		answer = answer and answer[1]
		
		if answer == "change":
			cur = self["config"].getCurrent()
			
			if cur:
				sel = self["config"].getCurrentIndex()
				self.session.openWithCallback(self.createConfigEntry, Init7ChannelLoaderCountrySelector, self.countryAndCodeList, config.plugins.init7channelloader.countries[sel].value, sel)
		elif answer == "remove":
			cur = self["config"].getCurrent()
			
			if cur:
				sel = self["config"].getCurrentIndex()
				config.plugins.init7channelloader.countrycount.value -= 1
				config.plugins.init7channelloader.countrycount.save()
				del config.plugins.init7channelloader.countries[sel]
				config.plugins.init7channelloader.countries.save()
				config.plugins.init7channelloader.save()
				configfile.save()
				
		self.createConfig()			
			
class Init7ChannelLoaderCountrySelector(Screen, HelpableScreen, ConfigListScreen):
	skin = """
			<screen position="center,center" size="1240,100" title="Init7 Channel Loader - Select Country">
				<widget name="config" position="0,0" size="1240,50" scrollbarMode="showOnDemand" separation="70" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/red.png" position="10,60" zPosition="3" size="30,30" transparent="1" name="red" />
				<widget name="ButtonRedText" position="50,60" size="250,30" valign="center" halign="left" font="Regular;25" transparent="1" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/green.png" position="310,60" zPosition="3" size="30,30" transparent="1" name="green" />
				<widget name="ButtonGreenText" position="350,60" size="250,30" valign="center" halign="left" font="Regular;25" transparent="1" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/yellow.png" position="610,60" zPosition="3" size="30,30" transparent="1" name="yellow" />
				<widget name="ButtonYellowText" position="650,60" size="250,30" valign="center" halign="left" font="Regular;25" transparent="1" />
				<ePixmap alphatest="blend" pixmap="skin_default/buttons/blue.png" position="910,60" zPosition="3" size="30,30" transparent="1" name="blue" />
				<widget name="ButtonBlueText" position="950,60" size="250,30" valign="center" halign="left" font="Regular;25" transparent="1" />
			</screen>	
	"""
	
	def __init__(self, session, countryList, currentCountry=None, currentIndex=None, args=None):
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self.session = session
		self.countryList = countryList
		self.currentIndex = currentIndex
		
		self["WizardActions"] = HelpableActionMap(self, "WizardActions",
		{
			"ok":		(self.createEntry, _("Create entry")),
			"back":		(self.cancel, _("Cancel")),
			
		}, -1)
		
		self["ColorActions"] = HelpableActionMap(self, "ColorActions",
		{
			"green":	(self.createEntry, _("Create entry")),
			"red":		(self.close, _("Cancel")),
		}, -1)
	
		self["ButtonGreenText"] = Label(_("Create entry"))
		self["ButtonRedText"] = Label(_("Cancel"))
		self["ButtonYellowText"] = Label()
		self["ButtonBlueText"] = Label()		
		
		self.list = []
		
		defaultValue = "none"
		if currentCountry is not None:
			defaultValue = currentCountry
		self.countryEntry = NoSave(ConfigSelection(default=defaultValue, choices = self.countryList + [ ("none", _("None")) ] ))
		self.list.append(getConfigListEntry(_("Country"), self.countryEntry ))
		
		ConfigListScreen.__init__(self, self.list, session=session)
		
		self["config"].setList(self.list)

		if self.currentIndex is not None:		
			self.setLabelText()
			
		self.setTitle(_("Init7 Channel Loader - Select Country"))
		
	def setLabelText(self):
		self["ButtonGreenText"].setText(_("Update entry"))
		
	def createEntry(self):
		if self.countryEntry.value != "none":
			if self.currentIndex is None:
				config.plugins.init7channelloader.countries.append(ConfigSelection(default= self.countryEntry.value, choices=[ (self.countryEntry.value, self.countryEntry.getText())]))
				config.plugins.init7channelloader.countrycount.value = len(config.plugins.init7channelloader.countries)
			else:
				config.plugins.init7channelloader.countries[self.currentIndex].value = self.countryEntry.value
		
			config.plugins.init7channelloader.save()
			configfile.save()
			self.close(True)
		else:
			self.close(False)
			
	def cancel(self):
		self.close(False)
		
		
	